from .PasswordInput import PasswordInput
from .PlotlyOutput import PlotlyOutput
from .CpfInput import CpfInput
from .LinkOutput import LinkOutput
from .CheckboxInput import CheckboxInput
from .PhoneInput import PhoneInput
from .VideoInput import VideoInput
from .CurrencyInput import CurrencyInput
from .CameraInput import CameraInput
from .CustomInput import CustomInput
from .PandasRowSelectionInput import PandasRowSelectionInput
from .TextareaInput import TextareaInput
from .TimeInput import TimeInput
from .RichTextInput import RichTextInput
from .CodeInput import CodeInput
from .DateInput import DateInput
from .CnpjInput import CnpjInput
from .ImageInput import ImageInput
from .ListInput import ListInput
from .FileOutput import FileOutput
from .ToggleInput import ToggleInput
from .FileInput import FileInput
from .ChecklistInput import ChecklistInput
from .TextOutput import TextOutput
from .EmailInput import EmailInput
from .NumberSliderInput import NumberSliderInput
from .LatexOutput import LatexOutput
from .ImageOutput import ImageOutput
from .ProgressOutput import ProgressOutput
from .CardsInput import CardsInput
from .DropdownInput import DropdownInput
from .PandasOutput import PandasOutput
from .TagInput import TagInput
from .MultipleChoiceInput import MultipleChoiceInput
from .IframeOutput import IframeOutput
from .HtmlOutput import HtmlOutput
from .TextInput import TextInput
from .NumberInput import NumberInput
from .MarkdownOutput import MarkdownOutput
from .NpsInput import NpsInput
from .RatingInput import RatingInput



from ..widget_base import Output


class LinkOutput(Output):
    type = "link-output"

    def __init__(self, link_url: str, **kwargs):
        self.set_props(dict(link_url=link_url, **kwargs))

    def set_props(self, props):
        self.link_url = props.get("link_url", "")
        self.link_text = props.get("link_text", "Click here")
        self.same_tab = props.get("same_tab", False)
        self.full_width = props.get("full_width", False)

    def render(self, ctx: dict):
        return {
            "type": self.type,
            "linkText": self.link_text,
            "linkUrl": self.link_url,
            "sameTab": self.same_tab,
            "fullWidth": self.full_width,
        }



import sys
from typing import Callable, Literal

from abstra_internals.stdio_skip_request import ESCAPE_STRING
from .execution.execution import Execution

_og_stdout_write = sys.stdout.write
_og_stderr_write = sys.stderr.write


def writeWraper(
    type: Literal["stdout", "stderr"],
    write: Callable,
    text: str,
    print_exceptions: bool,
):
    try:
        write(str(text))
        skip_sending = text.startswith(ESCAPE_STRING)
        execution = Execution.get_current_execution()

        if execution and not skip_sending:
            execution.stdio(type, text)
    except Exception as e:
        if print_exceptions:
            write(f"[WRITE_WRAPPER] ERROR: {str(e)}\n\n")
    finally:
        return len(text)


def override_stdio(print_exceptions: bool):
    sys.stdout.write = lambda text: writeWraper(
        "stdout", _og_stdout_write, text, print_exceptions
    )
    sys.stderr.write = lambda text: writeWraper(
        "stderr", _og_stderr_write, text, print_exceptions
    )


def reset():
    sys.stdout.write = _og_stdout_write
    sys.stderr.write = _og_stderr_write



from ....repositories.stage_run import StageRun
from ....repositories.project.project import ConditionStage, WorkflowTransition
from warnings import warn


def match(workflow_transition: WorkflowTransition, variable_value):
    if workflow_transition.type != "conditions:patternMatched":
        return False
    if type(variable_value) == int or type(variable_value) == float:
        return workflow_transition.condition_value == str(variable_value)
    if type(variable_value) == str:
        return workflow_transition.condition_value == variable_value
    elif variable_value is None:
        return (
            workflow_transition.condition_value is None
            or workflow_transition.condition_value == ""
        )
    else:
        warn(f"Unknown type {type(variable_value)} for variable_value")
        return False


def condition_strategy(
    stage: ConditionStage,
    stage_run: StageRun,
    transition_type="conditions:patternMatched",
):
    variable_value = stage_run.data.get(stage.variable_name)

    next_stage_ids = [
        t.target_id for t in stage.workflow_transitions if match(t, variable_value)
    ]

    return [dict(stage=stage_id, data=stage_run.data) for stage_id in next_stage_ids]



from unittest import TestCase
from tests.fixtures import init_dir, clear_dir
from .project import ProjectRepository, ScriptStage


class ProjectTests(TestCase):
    def setUp(self):
        self.root = init_dir()

    def tearDown(self) -> None:
        clear_dir(self.root)

    def test_delete_file_if_remove_file_true(self):
        project = ProjectRepository.load()
        file = self.root / "script.py"
        file.write_text("print('hello, world')")

        script = ScriptStage(
            file="script.py",
            id="test",
            is_initial=True,
            title="test",
            workflow_position=(0, 0),
            workflow_transitions=[],
        )
        project.scripts.append(script)
        ProjectRepository.save(project)
        project.delete_stage("test", remove_file=True)
        self.assertFalse(file.exists())

    def test_dont_delete_file_if_remove_file_false(self):
        project = ProjectRepository.load()
        file = self.root / "script.py"
        file.write_text("print('hello, world')")

        script = ScriptStage(
            file="script.py",
            id="test",
            is_initial=True,
            title="test",
            workflow_position=(0, 0),
            workflow_transitions=[],
        )
        project.scripts.append(script)
        ProjectRepository.save(project)
        project.delete_stage("test", remove_file=False)
        self.assertTrue(file.exists())



from unittest import TestCase
from .threaded import threaded
from typing import Any


class ThreadedTest(TestCase):
    def test_threaded_with_exception(self):
        @threaded
        def func():
            raise Exception("test")

        thread = func()

        self.assertRaises(Exception, thread.result)

    def test_threaded_with_return(self):
        @threaded
        def func() -> Any:
            return "test"

        thread = func()

        self.assertEqual(thread.result(), "test")
